package com.jzq.cinpre.utils;

import com.jzq.cinpre.entity.CinLine;
import com.jzq.cinpre.entity.CinPoint;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Dji {
    public static int MaxValue = 100000;

    public List<CinPoint> getPath(List<CinPoint> pointList, List<CinLine> lineList,CinPoint start,CinPoint end){
        //初始化邻接矩阵
        int n = pointList.size();
        int[][] distance = new int[n][n];
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                distance[i][j] = MaxValue;
            }
        }

        //矩阵下标到点id的映射
        CinPoint[] id = new CinPoint[pointList.size()];
        //点id到矩阵下标的映射
        Map<Integer,Integer> map = new HashMap<>();
        int a=0;

        for(CinPoint cinPoint:pointList){
            int pid = cinPoint.getId();
            id[a] = cinPoint;
            map.put(pid,a++);
        }

        //构建邻接矩阵
        for(CinLine cinLine:lineList){
            int index1 = map.get(cinLine.getPId());
            int index2 = map.get(cinLine.getPpId());
            int wei = cinLine.getWeight();
            distance[index1][index2] = wei;
            distance[index2][index1] = wei;
        }

        int startindex = map.get(start.getId());
        int endindex = map.get(end.getId());

        //存放起点到其他节点的距离
        int[] result = new int[n];
        //节点是否被访问过
        boolean[] visited = new boolean[n];
        //路径
        ArrayList<Integer> path = new ArrayList<>();
        path.add(startindex);

        for(int i=0;i<n;i++){
            result[i] = distance[startindex][i];
        }

        while(true){
            //找到剩余未访问的节点中，距离start最近的一个节点
            int curNode = getNearestNode(distance, visited, result);
            //更新其余点距start,curNode的距离
            update(distance,curNode,visited,result,path);

            if(curNode == endindex){
                break;
            }
        }

        List<CinPoint> res = new ArrayList<>();

        for(int node:path){
            res.add(id[node]);
        }
        return res;
    }

    private int getNearestNode(int[][] distance, boolean[] visited, int[] result){
        int index = -1;
        int min = MaxValue;
        for(int i=0;i<distance.length;i++){
            if(!visited[i]&&result[i]!=-1&&result[i]<min){
                index = i;
                min = result[i];
            }
        }

        return index;
    }

    private void update(int[][] distance, int curNode, boolean[] visited, int[] result, ArrayList<Integer> path){
        visited[curNode] = true;
        path.add(curNode);

        for(int j=0;j<distance.length;j++){
            // 节点j未访问过 && 节点curNode与节点j之间需要直接相连
            if(!visited[j]&&distance[curNode][j]!=-1){
                if (result[j] > result[curNode] + distance[curNode][j] || result[j] == -1) {
                    result[j] = result[curNode] + distance[curNode][j];
                }
            }
        }
    }
}


